<?php

require_once __DIR__ . '/protocolHeaderPartParser.php';

/**
 * runs the given black box test against the given files
 * @param string $mainFileNameWithExtension the main file to compile (with extension)
 * @param $test
 * @param $fullWorkingDirPath
 * @param int|string $timeout the timeout in MS  (can be string from db or int from default values)
 * @param int|string $compileTimeout the compile timeout in MS  (can be string from db or int from default values)
 * @param int|string $memoryLimit the memory limit in kb  (can be string from db or int from default values)
 * @param int|string $diskSpaceLimit the max disk space the program can write to (can be string from db or int from default values)
 * @param string $compileCmd the command to execute to compile the file(s)
 * @param string $execCmd the command to execute to run the test
 * @param $sourceFileExtensions array the list with the file extensions from the source files for the p language
 * @param $needCompilation bool true: should compile sources, false: not (this assumes the sources are already compiled)
 * @param $maxLinesToRead int the max lines to read from the user program (passed to the runner)
 * @param $maxErrLinesToRead int the max lines to read from the user program (from stderr) (passed to the runner)
 * @param $maxLinesToWrite int the max lines to write the user program (passed to the runner)
 * @param $justRun true: just run test, false: normal back box test
 * @param $uniqueSessionId string the unique session id for the test runner to handle sandbox file system management
 * @param $showTestRunnerDebugOutput 1: show internal debug output from the test runner (included in the test response)
 * @param $compilerOptions string the options for the compiler
 * @return array the result
 * @internal param array $allFiles all files to copy in teh dir to use
 * Format: array[int] => {fileName: string, fileContent: string}
 */
function do_blackBoxTest($mainFileNameWithExtension, $test, $fullWorkingDirPath,
                         $timeout, $compileTimeout, $memoryLimit, $diskSpaceLimit,
                         $compileCmd, $execCmd, $sourceFileExtensions, $needCompilation,
                         $maxLinesToRead, $maxErrLinesToRead, $maxLinesToWrite,
                         $justRun,
                         $uniqueSessionId,
                         $showTestRunnerDebugOutput,
                         $compilerOptions


)
{

    global $isDebug;
    global $config;
    global $s_command_blackBoxTest;
    global $s_command_justRunTest;
    global $s_test_content;
    global $s_test_allAssets;
    global $s_return_val;
    global $s_output;
    global $s_user_program_exit_code;
    global $s_runnerVersion;
    global $s_timeForCompiling;
    global $s_timeForUserProgram;
    global $s_timeoutInMsUsed;
    global $s_compileTimeoutInMsUsed;
    global $s_output_without_header_part;


    $command_to_execute_string = $s_command_blackBoxTest;

    if ($justRun) {
        $command_to_execute_string = $s_command_justRunTest;
    }


    # compile is done in the runner
    $commandToExecute = $execCmd;
    $output = [];
    $return_var = -1;

    $testContent = $test[$s_test_content];
    $allAssets = $test[$s_test_allAssets];

    # create the assets for the current test
    if (createTestAssets($allAssets, $fullWorkingDirPath) === FALSE) {
        //error message is in createFiles function
        return array(
            $s_return_val => 100,
            $s_output => 'could not create asset(s)',
            $s_user_program_exit_code => NULL,
            $s_runnerVersion => NULL,
            $s_timeForCompiling => NULL,
            $s_timeForUserProgram => NULL,
        );
    }


    $longCmd = $config['runner']
        . ' ' . escapeshellarg($command_to_execute_string)                              # arg[0] the test method
        . ' ' . escapeshellarg($fullWorkingDirPath)                               # arg[1] dir path
        . ' ' . escapeshellarg($mainFileNameWithExtension)                        # arg[2] file path
        . ' ' . escapeshellarg($compileCmd)                                       # arg[3] compile command

        . ' ' . escapeshellarg($commandToExecute)                                # arg[4] command to execute the test
        . ' ' . escapeshellarg($timeout)                                          # arg[5] the timeout in ms
        . ' ' . escapeshellarg($compileTimeout)                                   # arg[6] the compile timeout in ms
        . ' ' . escapeshellarg($memoryLimit)                                      # arg[7] the memory limit
        . ' ' . escapeshellarg($diskSpaceLimit)                                   # arg[8] the disk limit
        . ' ' . escapeshellarg(implode(',', $sourceFileExtensions))         # arg[9] the source file extensions
        . ' ' . escapeshellarg($needCompilation ? 'true' : 'false')          # arg[10] true: compile program new, false: not
        . ' ' . escapeshellarg($maxLinesToRead)                                   # arg[11] max lines to read from the user program (inclusive)
        . ' ' . escapeshellarg($maxErrLinesToRead)                                # arg[12] max lines to read from the user program stderr (inclusive)
        . ' ' . escapeshellarg($maxLinesToWrite)                                  # arg[13]  max lines to write to the user program (inclusive)
        . ' ' . escapeshellarg($uniqueSessionId)                                  # arg[14] a unique session id
        . ' ' . escapeshellarg(($showTestRunnerDebugOutput === TRUE ? 1 : 0))     # arg[15] showTestRunnerDebugOutput
        . ' ' . escapeshellarg($compilerOptions)                                  # arg[8] options for the compiler

    ;

    if ($isDebug) {
        debug("using runner: " . $config['runner']);
        debug("full command: " . $longCmd);
    }

    # the test content is passed in as stdin (because might be too long for a console argument
    # e.g. windows its 32bit ca. 2.000 kb...

    $pipesDescriptor = array(
        0 => array('pipe', 'r'), # stdin is a pipe that the child will read from
        1 => array('pipe', 'w'),  # stdout is a pipe that the child will write to
        2 => array("pipe", "w")   # stderr
    );


    if ($isDebug) {
        $time_pre = microtime(true);
    }


    $env = $config['environmentVarsParsed'];

    # without bypass_shell it won't work on windows
    $process = proc_open($longCmd, $pipesDescriptor, $pipes, $fullWorkingDirPath, $env, array('bypass_shell' => TRUE));

//    $state = proc_get_status($process);
//    warn('open pid: ' . $state['pid']);

    if (is_resource($process)) {
        fwrite($pipes[0], $testContent);
        fclose($pipes[0]);


        $output = stream_get_contents($pipes[1]);
        fclose($pipes[1]);

        $errorOutput = stream_get_contents($pipes[2]);
        fclose($pipes[2]);

        $return_var = proc_close($process);
    }

    # we cannot use exec because this does no let us write to the stdin of the process
    #exec($longCmd, $output, $return_var);

    if ($isDebug) {
        $time_post = microtime(true);
        $exec_time = ($time_post - $time_pre) * 1000; //in ms
        debug("time to run the black-box-tests: " . $exec_time);
    }

    if ($isDebug && (isset($errorOutput) && trim($errorOutput) !== '')) {
        debug("error during execution of the (blackbox) test runner: " . $errorOutput);
    }


    # the test assets are removed when the dir gets removed
    # also the test runner could renamed/added/deleted some files

    $headerPart = parseHeaderPart($output);

    //$output

    #file_put_contents($fullWorkingDirPath . 'xyz.txt', $output);

    # in case the test runner didn't even run we may need to cast (vals from db)
    if (gettype($timeout) === 'string') {
        $timeout = intval($timeout, 10);
    }

    if (gettype($compileTimeout) === 'string') {
        $compileTimeout = intval($compileTimeout, 10);
    }

    return array(
        $s_return_val => $return_var,
        $s_output => $headerPart[$s_output_without_header_part],
        $s_user_program_exit_code => $headerPart[$s_user_program_exit_code],
        $s_runnerVersion => $headerPart[$s_runnerVersion],
        $s_timeForCompiling => $headerPart[$s_timeForCompiling],
        $s_timeForUserProgram => $headerPart[$s_timeForUserProgram],
        $s_timeoutInMsUsed => $timeout,
        $s_compileTimeoutInMsUsed => $compileTimeout,
    );
}
